import {serviceTypes, services, baseURL, sealKey, apiPath} from "./constants/generalConstants.js";
import exceptionalError from "exceptionalhandler";
import {getAllSubsApiResp} from './constants/apiResps.js'
import {reqParams} from './constants/generalConstants.js'
import fetch, {Response} from 'node-fetch'


export async function performSealRequest(reqType: serviceTypes, arg: null|string = null){
    if (!(reqType in serviceTypes)){
        throw new exceptionalError(`Unknown API req type ${reqType}`)
    }

    let reqParams = setReqParams(reqType, arg)
    let parsedResponse = await doTheApiThing(reqParams, reqType)
    if(reqType === serviceTypes.allSubs){
        await checkPagination(parsedResponse, reqParams, reqType);
    }
    return parsedResponse;
}

async function checkPagination(parsedResponse: getAllSubsApiResp, ogReqParams: reqParams, reqType: serviceTypes){
    if (!parsedResponse.payload.page){
        throw new exceptionalError(`Somehow ended up with a subs response w/ no pages?`)
    }
    
    while(parsedResponse.payload.page < parsedResponse.payload.total_pages){
        let newReqDetails:reqParams = {
            url: `${ogReqParams.url}?page=${parsedResponse.payload.page + 1}`,
            fetchOpts: ogReqParams.fetchOpts
        }

        let newResp = await doTheApiThing(newReqDetails, reqType) as getAllSubsApiResp;
        if(newResp.payload && newResp.payload.subscriptions && newResp.payload.page){
            for (const subDeets of newResp.payload.subscriptions){
                parsedResponse.payload.subscriptions.push(subDeets)
            }
            parsedResponse.payload.page = newResp.payload.page
        } else {
            parsedResponse.payload.page = 999999
            break;
        }
    }
}

async function doTheApiThing(reqParams: reqParams, reqType: serviceTypes){
    let req: Response;
    try{
        req = await fetch(reqParams.url, reqParams.fetchOpts)
    } catch(e){
        throw e;
        // maybe handle this better later
    }
    let resp = await handleResponse(req, reqType);
    return resp
}

function setReqParams(reqType: serviceTypes, arg: string|null): reqParams{
    const apiReqDeets = services[reqType];

    let reqUrl = `${baseURL}${apiPath}${apiReqDeets.endPoint}`
    if(apiReqDeets.argStr){
        if(!arg){
            throw new exceptionalError(`Arg '${apiReqDeets.argStr}' is required for ${reqType}`)
        }
        reqUrl = reqUrl.replace(apiReqDeets.argStr, arg);
    } else {
        if(arg){
            throw new exceptionalError(`${reqType} has no args but you provided "${arg}"`)
        }
    }
    
    return {
        url: reqUrl,
        fetchOpts: {
            method: apiReqDeets.method,
            headers: {
                'X-Seal-Token': sealKey
            }
        }
    }
}

async function handleResponse(apiResp: Response, reqType: serviceTypes){
    switch(apiResp.status){
        case 200:
            let body = await apiResp.json() as any;
            if(!body.success){
                let errMsg = '';
                if(!body.error){
                    errMsg = 'No Error Provided'
                } else {
                    errMsg = body.error
                }
                throw new exceptionalError(`Seal API Req ${reqType} returned ${apiResp.status} but success was false with error "${errMsg}"`)
            }
            return body;
            break;
        default:
            let errMsg = '';
            if(apiResp.body){
                errMsg = getBodyErrMsg(await apiResp.text());
            }
            throw new exceptionalError(`Seal API Req ${reqType} returned ${apiResp.status} w/ body "${errMsg}"`)
            break;
    }
}

function getBodyErrMsg(bodyText: string){
    const errRegex = /error":"(.+?)"/i;
    let retStr = '';
    if(bodyText.match(errRegex)){
        retStr = `Error: ${bodyText.match(errRegex)![1]}`
    }
    return retStr;
}